#!/bin/bash

debug() {
	[ "${debug:-0}" = '1' ] && log "$@" || true
}

zero_files() {
	for file in "$@"; do
		> $LISTDIR/$file
	done
}

URL_FILES=( http.load http.noload http_full.noload )
URL_BINDINGS=(
	"list http.load"
	"whitelist http.noload"
	"fullwhitelist http_full.noload"
)

IP_FILES=( ip.load ip.noload ip6.load ip_block.load system.iplist system.ip6list )
IP_BINDINGS=(
	"iplist ip.load"
	"ip6list ip6.load"
	"whiteiplist ip.noload"
	"ip_block ip_block.load"
)

whitelisting() {
	for file in "$1" "$2"; do
		[ "$file" == '/dev/null' ] && continue
		LC_CTYPE='' LANG='' sort -u "$file" > "$file.sort"
		mv -f "$file.sort" "$file"
	done
	LC_CTYPE='' LANG='' join -a 1 -v 1 -j 1 "$2" "$1" > "$2.clean"
	mv -f "$2.clean" "$2"
}

categorize() {
	debug categorize files_mask="$1" output="$2"
	find $LISTDIR/ -type f -name "*.$1" -exec egrep -v '^(#.*|)$' {} \; | LC_CTYPE='' LANG='' sort -u > "$LISTDIR/$2"
	debug post categorize files_mask="$1" output="$2"
}

remove_private_networks() {
	egrep -v "^1(27|92[.]168|0[.]|72[.](1[6-9]|2[0-9]3[01]))[0-9.]*$" $1 > $1.no_private || true
	mv -f $1.no_private $1
}

unificate_https_domains() {
	grep ^https:// $LISTDIR/$1 > $TMPDIR/$1.links || true
	grep -v ^https:// $LISTDIR/$1 > $TMPDIR/$1.domains || true
	cut -d '/' -f3 $TMPDIR/$1.links | sort -u >> $TMPDIR/$1.domains
	mv -f $TMPDIR/$1.domains $LISTDIR/$1
}

# разбираемся с https
resolv() {
	categorize 'httpslist' system.https
	categorize 'whitehttpslist' system.nohttps
	if [ ! -s $LISTDIR/system.https ]; then
		echo '- Пропускаем resolv, так как system.https пустой'
		return 0
	fi
	unificate_https_domains system.https
	unificate_https_domains system.nohttps
	whitelisting $LISTDIR/system.nohttps $LISTDIR/system.https
	sort -u $LISTDIR/system.https | egrep -v "^$ip_regex$" > $LISTDIR/https.resolv || true
	if [ "${1:-fast}" = 'slow' ]; then
		echo '- Обновляем базу https'
		IPV6="${filter['ipv6']:-0}" $BINDIR/ip_update.sh
		echo '- Забираем из неё список нужных адресов'
	fi
	$BINDIR/nslookuper.sh 'ip' > $LISTDIR/system.iplist
	if [ "${filter['ipv6']:-0}" = '1' ]; then
		$BINDIR/nslookuper.sh 'ip6' > $LISTDIR/system.ip6list
	fi
	#TODO: remove_private_networks_ipv6
	remove_private_networks $LISTDIR/system.iplist
}

# добавляем из минюста и собственных списков
categorize_url() {
	debug categorize
	for binding in "${URL_BINDINGS[@]}"; do
		categorize $binding.all
	done
}


# добавляем из минюста и собственных списков
categorize_ip() {
	debug categorize
	for binding in "${IP_BINDINGS[@]}"; do
		categorize $binding
	done
}

# при необходимости проходимся по белым спискам и вырезаем из полного списка
whitelisting_url() {
	debug whitelisting
	whitelisting $LISTDIR/http.noload $LISTDIR/http.load &
	whitelisting_fulldomain $LISTDIR/http_full.noload $LISTDIR/http.load &
	for list in $(find $HOOKDIR/custom_url_lists/ -type f); do
		whitelisting /dev/null $list &
	done
	wait
}

# при необходимости проходимся по белым спискам и вырезаем из полного списка
whitelisting_ip() {
	debug whitelisting
	whitelisting $LISTDIR/ip.noload $LISTDIR/ip.load
}
