#!/bin/bash

. /usr/local/Reductor/etc/const

set -eu

wrapper() {
	local label="$1"
	local funcname="$2"
	shift 2
	if [ -z "$file" ]; then
		local file="$3"
		shift
	fi
	echo "# $label:"
	$funcname "$@" "$file"
	echo
}

__tcpdump() {
	tcpdump -nee -r $1 2>/dev/null
}

gather() {
	[ "$gather" = '0' ] && return
	tcpdump -nneei $dev -c $count tcp port not 22 -w "$1" 2>/dev/null
}

dump_mac_addrs() {
	__tcpdump "$file" | awk '{print $4}' | grep -vi "ff:ff:ff:ff:ff:ff" | sort -u | tr -d ','
}

mirror_type_detect() {
	local link_mac_addr
	local dump_mac_addr
	local dump_mac_addrs_count

	dump_mac_addrs > $TMPDIR/dump_mac_addrs
	link_mac_addr="$(ip l show eth0 | egrep -o '([A-Za-z0-9]{2}:){5}[A-Za-z0-9]{2}' | head -1)"
	dump_mac_addrs_count="$(wc -l < $TMPDIR/dump_mac_addrs)"


	if [ "$dump_mac_addrs_count" = '1' ]; then
		read dump_mac_addr < $TMPDIR/dump_mac_addrs
		if [ "$dump_mac_addr" = "$link_mac_addr" ]; then
			echo "Скорее всего зеркало трафика L3"
		else
			echo "На $dev отправляется зеркало трафика на 1 MAC-адрес"
			echo "Адрес $dev - $link_mac_addr, а приходит $dump_mac_addr"
			echo "Это немного странно."
		fi
	elif [ "$dump_mac_addrs_count" -gt 1 ]; then
		echo "Скорее всего зеркало трафика - L2"
	else
		echo "Не удалось определить тип зеркала трафика"
	fi
	rm -f $TMPDIR/dump_mac_addrs
}

protos() {
	tcpdump -qnnee -r $1 2>/dev/null | cut -d ' ' -f5 | sort -u | tr -d ','
}

tags() {
	__tcpdump $1 | egrep -o "vlan [0-9]+" | sort -u
}

ipvX() {
	__tcpdump $1 | grep -o "IPv[46]" | sort -u
}

vpn() {
	__tcpdump $1 | egrep -o 'ethertype PP[^ ]+' | sort -u
}

proto_analyze_parse() {
	local src_count="$1"
	local dst_count="$2"
	local result="$3"
	local proto="$4"
	local port="$5"
	echo "Пакетов с $proto src port $port (всего $count)"
	echo $src_count
	echo
	echo "Пакетов с $proto dst port $port (всего $count)"
	echo $dst_count
	echo
	if [ "$src_count" -gt "$dst_count" ]; then
		echo "## WARNING"
		echo "Пакетов с src port $port больше, чем с dst port $port, скорее всего зеркало настроено задом наперёд"
		echo
		return 1
	fi
	return 0

}

proto_analyze_human() {
	local rc=0
	params="$(proto_analyze "$@")" || rc=$? || true
	proto_analyze_parse $params
	return $rc
}

proto_analyze() {
	local proto=$1
	local port=$2
	local result=0
	if [ -z "$file" ]; then
		local file=$3
	fi
	src_count="$(tcpdump -nn -r $file $proto src port $port 2>/dev/null | wc -l)"
	dst_count="$(tcpdump -nn -r $file $proto dst port $port 2>/dev/null | wc -l)"
	[ "$src_count" -le "$dst_count" ] || result=$? || true
	[ "$dst_count" != 0 ] || result=$? || true
	echo $src_count $dst_count $result $proto $port
	return $result
}

__own_pkts() {
	ip="$(ip r g 8.8.8.8 | egrep -o src.* | egrep -o $ip_regex)"
	[ "$(tcpdump -nn -r $file host $ip 2>/dev/null | wc -l)" = 0 ]
}

own_pkts() {
	local rc=0
	__own_pkts || rc=$?
	[ "$rc" == 0 ] && echo "Отсутствует" || echo "Присутствует"
	return $rc
}

main() {
	gather "$3"
	wrapper "Тип зеркала трафика" mirror_type_detect
	wrapper "Протоколы в зеркале (без разворачивания)"  protos
	wrapper "Тэги vlan" tags
	wrapper "Версии IP-протокола" ipvX
	wrapper "Пакеты VPN" vpn
	wrapper "HTTP" proto_analyze_human tcp 80
	wrapper "HTTPS" proto_analyze_human tcp 443
	wrapper "DNS" proto_analyze_human udp 53
	wrapper "Наличие IP с которым уходим в default route в зеркале" own_pkts
}

include=0
gather=1
[ "${1:-}" = include ] && include=1 && shift
[ "${1:-}" = nogather ] && gather=0 && shift
dev="${1:-eth1}"
count="${2:-1000}"
file="${3:-/tmp/mirror.pcap}"

if [ "$include" != '1' ]; then
	echo main "$dev" "$count" "$file"
	main "$dev" "$count" "$file"
fi
