#!/usr/bin/env bash

. /usr/local/Reductor/etc/const

LOCKFILE_URL=$LOCKDIR/url.lock
LOCKFILE_DOMAIN=$LOCKDIR/domain.lock
LOCKFILE_IP=$LOCKDIR/ip.lock
LOCKFILE_PORT=$LOCKDIR/port.lock

FD_URL=3
FD_DOMAIN=4
FD_IP=5
FD_PORT=6

declare -a urls=( url_{http,https} ) # незачем обрабатывать unknown!
declare -a domains=( domain_{exact,mask,proxy,hsts} )
declare -a ip=( ip{,6}_{https{,_plus},block,port} )
declare -a ports=( port_http port_https )
declare -a actions=( categorize normalize whitelist )

prepare() {
	mkdir -p $LOCKDIR $LISTDIR/{load,tmp,resolver}
}

process() {
	local prefix="$1"
	for action in "${actions[@]}"; do
		log $BINDIR/$action.sh "$prefix"
		$BINDIR/$action.sh "$prefix"
	done
	start_use $prefix
}

start_use() {
	local prefix="$1"
	log ${FUNCNAME[0]} $prefix
	cp -a $LISTDIR/tmp/$prefix.load $LISTDIR/load/$prefix.load
}

cleanup() {
	find $LISTDIR/tmp/ $LISTDIR/load/ -type f -empty -delete
	release_lock $LOCKFILE_URL $FD_URL || true
	release_lock $LOCKFILE_DOMAIN $FD_DOMAIN || true
	release_lock $LOCKFILE_IP $FD_IP || true
	release_lock $LOCKFILE_PORT $FD_PORT || true
}

__custom_exit() {
	cleanup
}

show_list() {
	eval echo \"'${'$1'[@]}'\"
	exit
}

only_ip() {
	log "Обрабатываем только списки IP"
	set_lock $LOCKFILE_IP 60 $FD_IP
	for prefix in "${ip[@]}"; do
		process $prefix
	done
	release_lock $LOCKFILE_IP $FD_IP
	exit 0
}

main() {
	local prefix
	[ "$*" != ':' ] || return 0
	[ "$1" != '--show-list' ] || show_list "$2"
	[ "$1" != '--only-ip' ] || only_ip
	set -eu
	trap __exit EXIT
	prepare
	# because only urls take a long time to process
	set_lock $LOCKFILE_URL 60 $FD_URL
	find $LISTDIR/load/ -type f -name "url_*" -delete
	for prefix in "${urls[@]}"; do
		process $prefix
	done
	release_lock $LOCKFILE_URL $FD_URL
	set_lock $LOCKFILE_DOMAIN 60 $FD_DOMAIN
	find $LISTDIR/load/ -type f -name "domain_*" -delete
	set_lock $LOCKFILE_PORT 60 $FD_PORT
	find $LISTDIR/load/ -type f -name "ip_*" -delete
	set_lock $LOCKFILE_IP 60 $FD_IP
	find $LISTDIR/load/ -type f -name "port_*" -delete
	for prefix in "${domains[@]}" "${ports[@]}" "${ip[@]}"; do
		process $prefix
	done
	set_lock $LOCKFILE_URL 60 $FD_URL
	$BINDIR/prepare_lists_options.sh
	cleanup
}

main "$@"
