#!/usr/bin/env python2
# coding: utf-8

""" скрипт для определения необходимости совершения срочной выгрузки """

import os
from sys import argv
import logging
from calendar import timegm
import suds
from dateutil import parser
from dump_parser import main_local_dump_date

__author__ = 'Oleg Strizhechenko <oleg@carbonsoft.ru>'


class Dates(object):

    def __init__(self):
        self.endpoint = "http://vigruzki.rkn.gov.ru/services/OperatorRequest/?wsdl"
        self.date_server = None
        self.date_server_urgently = None
        self.responce = None
        self.date_local = None
        self.lookup_local()
        self.lookup_server()

    def lookup_server(self):
        """ узнаём, когда выкатили последний срочный апдейт """
        client = suds.client.Client(self.endpoint, cache=None)
        self.responce = client.service.getLastDumpDateEx()
        self.date_server = self.responce.lastDumpDate / 1000
        self.date_server_urgently = self.responce.lastDumpDateUrgently / 1000

    def lookup_local(self):
        """ выдёргиваем время, когда был сформирован локальный дамп """
        manual_run_path = './reductor_container/gost-ssl/php/dump.xml'
        dumpxml = os.getenv('DUMPXML', manual_run_path)
        update_time = main_local_dump_date(dumpxml)
        self.date_local = timegm(parser.parse(update_time).utctimetuple())

    def update_required(self):
        """ условия выгрузки РКН """
        # datetime.now().minute == 10
        return self.date_server > self.date_local

    def to_dict(self):
        return {
            'update_required': self.update_required(),
            'date_server': self.date_server,
            'date_server_urgently': self.date_server_urgently,
            'date_local': self.date_local,
            'obsolescence': self.date_server - self.date_local
        }

    def to_telegraf(self):
        """ dumpxml date_server=1482480000,obsolescence=14400,date_local=1482465600,update_required=1,date_server_urgently=1482432252 """
        output = ",".join("{0}={1}".format(k, int(v)) for k, v in self.to_dict().items())
        return "dumpxml {0}".format(output)


def setup_logger():
    log_level = logging.INFO
    if os.environ.get('log_level') == 'DEBUG':
        log_level = logging.DEBUG
    logging.basicConfig(format='%(asctime)-15s %(message)s', level=log_level)


def main():
    """ действуем как утилита, а не либа + логируем результат """
    setup_logger()
    dates = Dates()
    if '--telegraf' in argv:
        print dates.to_telegraf()
        exit(0)
    logging.info(dates.to_dict())
    exit(not dates.update_required())  # exit(not need_update()) == True == exit(0)


if __name__ == '__main__':
    main()
