#!/bin/bash

. /usr/local/Reductor/etc/const
# shellcheck disable=SC1090
. $CONFIG
# shellcheck disable=SC1090
. $MENULIB

ask_custom() {
	local question="$1"
	local param_config="$2"
	local defaultno=""
	if [ "$param_config" == "0" ];then
		defaultno="--defaultno"
	fi
	dialog --stdout $defaultno --yes-label "Да" --no-label "Нет" --yesno "$question" 10 50
}

question_choice() {
	local answer=0
	local question="$1"
	local config_value="$2"
	! ask_custom "$question" "$config_value" || answer=1
	echo $answer
}

get_prev_value() {
	local var="$1"
	/usr/local/Reductor/bin/inline_json /var/reg/CRB-Reductor.reg > /tmp/inline_reg
	( . /tmp/inline_reg; echo "${!var}" )
}

dialog_read_master() {
	dialog --title "$1" --ok-label "Далее" --nocancel --stdout --inputbox "$2" "${3:-0}" "${4:-0}" "${5:-}"
}


question_inputbox() {
	local title="$1"
	local question="$2"
	local regex="$3"
	local prev_value="$4"
	local optional="${5:-0}"
	local var

	while ! echo "${var:-}" | egrep -q "$regex"; do
		var="$(dialog_read_master "$title" "$question" 10 80 "$prev_value")"
		if [ "$optional" = '1' ] && [ "$var" = '' ]; then
			break
		fi
	done
	echo "$var"
}

ask_company_name() {
	local regex="^[^']+$"
	question_inputbox "Регистрация" "Введите название компании"  "$regex" "$(get_prev_value "company_name")"
}

ask_email_adress() {
	local email_regexp="^[a-z0-9_\+-]+(\.[a-z0-9_\+-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*\.([a-z]{2,4})$"
	local optional="${2:-0}"
	question_inputbox "Регистрация" "$1" "$email_regexp" "$(get_prev_value "$2")" $optional
}

ask_phone_number() {
	local optional=1
	question_inputbox "Регистрация" "$1" '[0-9]{8,11}' "$(get_prev_value "$2")" "$optional"
}

ask_inn() {
	# shellcheck disable=SC2154
	question_inputbox "Регистрация" "Введите ИНН (10-12 цифр)" '[0-9]{10,12}' "${autoupdate['inn']}"
}

ask_ogrn() {
	question_inputbox "Регистрация" "Введите ОГРН (13-15 цифр)" '[0-9]{13,15}' "${autoupdate['ogrn']}"
}

ask_url_redirect() {
	local url_regexp='(http|https)://[^|/"]+'
	question_inputbox "Настройка фильтрации" "URL для редиректа HTTP-запросов" "$url_regexp" "${filter['forbidden_url']}"
}

ask_local_url() {
	local url_regexp='(http|https)://[^|/"]+'
	question_inputbox "Настройка фильтрации" "Локальный URL для редиректа HTTP-запросов" "$url_regexp" "http://$($BINDIR/dnsenabled --ext-ip)/block.html"
}

admin_ip() {
	if [ "${admin['ip']}" = '' ]; then
		admin['ip']="$(who am i | awk '{print $5}' | tr -d '()')"
	fi
	# shellcheck disable=SC2154
	question_inputbox "Администрирование" "Перечислите через пробел список IP адресов администраторов, у них будет доступ к веб-интерфейсу" "$ip_regex" "${admin['ip']}"
}

admin_web_ip() {
	if [ "${admin['web_ip']}" = '' ]; then
		admin['web_ip']="$($BINDIR/dnsenabled --ext-ip)"
	fi
	question_inputbox "Администрирование" "IP адрес на котором будет доступен веб-интерфейс" "$ip_regex" "${admin['web_ip']}"
}


admin_web_port() {
	question_inputbox "Администрирование" "Порт на котором будет доступен веб-интерфейс" '[0-9]{1,5}' "${admin['web_port']}"
}

get_ip_list() {
	tmpfile=/tmp/ip_dns.list
	ip -4 -o a | cut -d ' ' -f 7 | grep -v "^127." | grep -v "^169.25.*/32" | cut -d '/' -f1 > $tmpfile
	while read ip; do
		echo -n "$ip '' '' "
	done < $tmpfile
	echo
	rm -f $tmpfile
}

choose_dns_ip() {
	local baseopts="--stdout --output-fd 2 --colors --trim --backtitle 'setup_master' --no-shadow --item-help --cancel-label 'Не знаю' --ok-label 'Выбрать' "
	local NO_WARRANTY="Мы не гарантируем стабильность её работы, поэтому рекомендуем после завершения тестирования установить заглушку на собственном сервере, либо прямо на Carbon Reductor"
	local ip_list
	local ip
	ip_list="$(get_ip_list) $CARBONSOFT_BLACKHOLE 'Заглушка CarbonSoft' 'Рекомендуется использовать только для тестирования'"
	ip="$(eval "dialog $baseopts --menu 'Выберите IP для DNS-спуфинга:' 11 50 11 $ip_list 3>&1 1>&2 2>&3")"
	if [ "$?" != '0' ] || [ "$ip" = 'Далее' ]; then
		ip="$CARBONSOFT_BLACKHOLE"
		show_msg "Так как вы включили DNS-модуль, но не выбрали IP адрес заглушки, то для демонстрации работы фильтрации DNS-трафика будет использоваться заглушка CarbonSoft.\n\n$NO_WARRANTY"
	elif [ "$ip" = "$CARBONSOFT_BLACKHOLE" ]; then
		show_msg "Вы выбрали заглушку CarbonSoft.\n\n$NO_WARRANTY"
	fi
	filter['dns_ip']="$ip"
}

select_rkn_update_method()
{
	local baseopts="--stdout --output-fd 2 --colors --trim --backtitle 'setup_master' --no-shadow --item-help --nocancel --ok-label 'Выбрать' "
	local variants="'pem' 'Генерация запроса и подписи на сервере.' 'Генерировать запрос и подпись на стороне сервера. Экспорт ключа. '\
		'xml' 'Использование готовых запроса и подписи.' 'Вы будете периодически подкладывать свои запрос и подпись, сгенеренную на windows-машине и проверенную вручную.'\
		'web' 'Скачивание с собственного веб-сервера.' 'Вы будете скачивать реестр периодически с собственного веб-сервера.'"
	option="$(eval "dialog $baseopts --menu 'Настройки выгрузок единого реестра' 12 75 10 $variants 3>&1 1>&2 2>&3")"
	echo "$option"
}
