#!/bin/bash

. /usr/local/Reductor/etc/const

get_defroute_dev() {
	ip r | grep -m1 default | egrep -o [a-z]+[0-9]+
}

get_iface_mac_list() {
	tmpfile=/tmp/iface_mac.list
	defroute_dev=$(get_defroute_dev)
	ip -o l | egrep ": (eth|em|en|bond)" | grep -v "@" | grep -vw "${defroute_dev// /}" > $tmpfile
	while read _ iface _ _ _ _ _ _ _ _ _ _ mac _; do
		echo -n "${iface%:} $mac '' "
	done < $tmpfile
	echo
	rm -f $tmpfile
}

remove_dev_from_all_bridges() {
	for bridge in $(egrep -o br[0-9]+ /proc/net/dev); do
		brctl delif $bridge $1 &>/dev/null
	done
}

replace_ifup() {
	mkdir -p $BACKUPDIR
	cp -v $NETWORK_SCRIPTS/ifup-eth "$BACKUPDIR/ifup-eth.$(date +%s)"
	cp -v $CONTRIB/$NETWORK_SCRIPTS/ifup-eth $NETWORK_SCRIPTS/ifup-eth
}

get_free_ip() {
	while true; do
		ip=169.255.255.$((RANDOM%254 + 1))
		ip_exist $ip || break
	done
	echo $ip
}

ip_exist() {
	ip a | fgrep -wqm1 "$1" && return 0
	fgrep -wqm1 -r "$1" $NETWORK_SCRIPTS
}

assert_mirror_devices()
{
	ifaces=$(get_iface_mac_list)
	[ -n "$ifaces" ] || terminate "Для сканирования трафика Вам необходимо установить дополнительное оборудование. Подробнее http://docs.carbonsoft.ru/pages/viewpage.action?pageId=51380324"
}

select_mirror_type()
{
	# shellcheck disable=SC2154
	local options="$baseopts --menu 'Выберите тип зеркала' 12 60 5"
	local variants="'l2_eth_only' 'L2-зеркало, без VLAN-тегов' 'Настраивается на коммутаторе' \
		'l2_vlan_only' 'L2-зеркало, весь трафик тегирован VLAN' 'Настраивается на коммутаторе' \
		'l2_vlan_mixed' 'L2-зеркало, часть с VLAN, часть без' 'Настраивается на коммутаторе' \
		'l3' 'L3-зеркало, прием на IP-адрес' 'Настраивается на маршрутизаторах, обычно с помощью xt_TEE' \
		'l3_vlan' 'L3-зеркало с настройкой VLAN' 'Настраивается на маршрутизаторах, обычно с помощью xt_TEE'"
	assert_mirror_devices
	eval dialog "$options" "$variants"
}

select_mirror_device() {
	mirror_type="$1"
	ifaces=$(get_iface_mac_list)
	while true; do
		# shellcheck disable=SC2154
		iface=$(eval "dialog $baseopts_eth --menu 'Добавить интерфейсы для сканирования трафика:' 11 70 11 $ifaces 'Далее' '' '' 3>&1 1>&2 2>&3")
		[ "$?" != '0' ] || [ "$iface" = 'Далее' ] && break
		setup_mirror_device "$iface" "$mirror_type" ${FUNCNAME[0]}
	done
}

select_mirror_device_vlan() {
	echo "Создаю vlan интерфейсы"
	ifaces=$(get_iface_mac_list)
	while true; do
		# shellcheck disable=SC2154
		iface=$(eval "dialog $baseopts_vlan --menu 'Добавить vlan на интерфейсы...' 11 70 11 $ifaces 'Далее' '' '' 3>&1 1>&2 2>&3")
		[ "$?" != '0' ] || [ "$iface" = 'Далее' ] && break
		setup_mirror_device $iface "$mirror_type" ${FUNCNAME[0]}
	done

}

l2_eth_only() {
	select_mirror_device ${FUNCNAME[0]}
	# shellcheck disable=SC2154
	misc['skipnetworkcheck']='0'
	save_and_exit
	return 0
}

l2_vlan_only() {
	select_mirror_device_vlan ${FUNCNAME[0]}
	misc['skipnetworkcheck']='0'
	save_and_exit
	return 0
}

l2_vlan_mixed() {
	talk_input_ip_then_vlan
	select_mirror_device ${FUNCNAME[0]}
	select_mirror_device_vlan "${FUNCNAME[0]}"
	misc['skipnetworkcheck']='0'
	save_and_exit
	return 0
}

l3() {
	select_mirror_device ${FUNCNAME[0]}
	misc['skipnetworkcheck']='1'
	save_and_exit
	return 0
}

l3_vlan() {
	select_mirror_device_vlan ${FUNCNAME[0]}
	# shellcheck disable=SC2034
	misc['skipnetworkcheck']='1'
	save_and_exit
	return 0
}

ask_tag_info() {
	local device="$1"
	local vlan_tag=""
	while ! echo "${vlan_tag:-}" | egrep -q "^[0-9]+$"; do
		vlan_tag="$(dialog_read "Введите номер тэга vlan:" 10 80)"
	done
	echo "$vlan_tag"
}

ask_ip_info() {
	local device="$1"
	local ip
	local netmask

	ip="$(dialog_read "Введите IP/маску для $device, например 192.168.100.15/24" 10 80)" || return 0
	local netmask
	if ! ipcalc -c -s -4 "$ip"; then
		talk_incorrect_ip_adress "$ip" && ask_ip_info "$@"
		return
	fi
	if [ "$ip" == "${ip////}" ]; then
		netmask=$(dialog_read "Введите маску для $device:") || return 0
		# netmask="$(ipcalc --netmask $ip/$netmask)"
		#netmask="${netmask##*=}"
	else
		netmask="$(ipcalc --prefix $ip)"
		netmask="${netmask##*=}"
		ip="${ip%%/*}"
	fi

	# на выходе мы обязаны получить запись вида 1.2.3.4/8
	# которую затем преобразует в конфиг setup_network_gen.sh
	echo  "$ip/$netmask"
}

setup_mirror_device() {
	local device="$1"
	local mirror_type="$2"
	local parent="$3" # select_mirror_device | select_mirror_device_vlan
	local tag='-'
	local ip_mask='-'

	ask_custom "Далее" "Отмена" "Добавить и настроить сканирование трафика с $device?" || return 0

	if [[ "$mirror_type" =~ (l2_vlan_only|l3_vlan) ]]; then
		tag="$(ask_tag_info "$device")"
	fi
	if [ "$mirror_type" == 'l2_vlan_mixed' ] && [ "$parent" = 'select_mirror_device_vlan' ]; then
		tag="$(ask_tag_info "$device")"
	fi
	if [[ "$mirror_type" =~ (l3|l3_vlan) ]]; then
		ip_mask="$(ask_ip_info $device)"
		remove_dev_from_all_bridges "$device"
	fi
	echo "$device ${tag:--} ${ip_mask:--}" >> $USERDIR/mirror_info.conf && device_is_set "$device"
}

# magik, tmplt & conf comes from param
genconfig() {
	for param in "$@"; do
		[[ "$param" = *=* ]] && eval $param
	done
	# shellcheck disable=SC2154
	while read line; do
		eval echo $line
	done < $tmplt > $conf
}

iplink2dev() {
	cut -d ' ' -f 2 | tr -d ':'
}

mirror_info_for_up_interfaces() {
	local question="Получить информацию о том, что сейчас приходит в поднятые интерфейсы, которые можно использовать в качестве зеркальных?"
	local defroute_dev
	local link_list
	declare -a link_list
	defroute_dev="$(get_defroute_dev)"
	link_list=( $(ip -o link | egrep ": (eth|em|en|bond)" | grep -v "@" | grep -vw "${defroute_dev// /}" | iplink2dev) )
	[ "${#link_list[@]}" -gt 0 ] || return 0
	ask_custom "Да" "Нет" "$question" || return 0
	clear
	for link in "${link_list[@]}"; do
		timeout -s 15 10s $BINDIR/mirror_info.sh "$link" 100 || true
	done
	read -p "PRESS ENTER TO CONTINUE..."
}
