#!/bin/bash

. /usr/local/Reductor/etc/const
. $CONFIG

set -eu

LOCKFILE_SIGNATURES=$LOCKDIR/signatures.lock
FD_SIGNATURES=7

prepare() {
	mkdir -p $TMPDIR $CACHEDIR
	params="$(get_reg_params)"
	URI="$URL/index.php?$params"
}

migrate_cache() {
	if egrep -q '^[0-9A-F]{10} ' $CACHEDIR/signatures.cache; then
		echo '- Немного изменяем формат кэша сигнатур'
		sed -E 's/^([0-9A-F]{10})( )/\1\t/g' -i $CACHEDIR/signatures.cache
	fi
}

curl_retry() {
	local res=1
	local retry=$1
	local timeout=$2
	shift 2
	while [ "$res" -gt 0 -a "$retry" -gt 0 ]; do
		curl -sS "$@" && return 0 || res=$?
		((retry--))
		[ "$retry" = 0 -a $res != 0 ] && return 1 || sleep $timeout
	done
}

show_unsigned_url() {
	touch $CACHEDIR/signatures.cache
	LC_CTYPE='' LANG='' sort -t '	' -u -k2 $CACHEDIR/signatures.cache > $TMPDIR/signed_url
	LC_CTYPE='' LANG='' sort -t '	' -u -k1 $1 > $TMPDIR/url_to_load
	LC_CTYPE='' LANG='' join -t '	' -a 1 -j1 1 -j2 2 -v 1 $TMPDIR/url_to_load $TMPDIR/signed_url
}

append_signatures() {
	local num=1
	[ "$loop_count" = '1' ] && echo -n '- Подгрузка недостающих сигнатур' || echo -n "."
	while read -r f; do
		echo -n "$((num++))=$(echo "$f" | base64 | tr '+' '_')&"
	done <<< "$(head -n 200 $TMPDIR/unsigned_url)" > $TMPDIR/curl_post_request
	if ! curl_retry 5 5 -sS -d "$(<$TMPDIR/curl_post_request)" "$URI" > $TMPDIR/new_signatures; then
		echo "Возникла проблема при обращении к серверу сигнатур"
		exit 0
	fi
	tabify $TMPDIR/new_signatures
	LC_CTYPE='' LANG='' sort -t '	' -u -k2 $CACHEDIR/signatures.cache $TMPDIR/new_signatures > $TMPDIR/merged_signatures 2>/dev/null
	LC_CTYPE='' LANG='' sort -t '	' -u -k2 $TMPDIR/merged_signatures > $CACHEDIR/signatures.cache
	rm -f $TMPDIR/merged_signatures $TMPDIR/new_signatures
}

main_loop() {
	prepare
	prev_count=0
	loop_count=1
	url_count=0
	clean=0
	for list in $(url_lists); do
		while show_unsigned_url $list > "$TMPDIR/unsigned_url"; do
			url_count=$(wc -l < $TMPDIR/unsigned_url)
			if [ "$url_count" -le '0' ]; then
				[ "$loop_count" -gt 1 ] && echo
				clean=0
				break
			fi
			if [ "$prev_count" = "$url_count" ]; then
				log "ERROR: loop stucked, skip get url_count=$url_count"
				head $TMPDIR/unsigned_url
				clean=1
				break
			fi
			prev_count=$url_count
			append_signatures
			# in case of stucked, ideal -doesn't called, coz need get <1000 signatures
			((loop_count++))
			[ "$loop_count" -lt 100 ] && sleep 0.05 || sleep 1
			[ "$loop_count" -gt 1000 ] && sleep 5
			[ "$loop_count" -gt 10000 ] && log "FATAL STUCKED" && sleep 60
		done
	done

	tabify $CACHEDIR/signatures.cache
	return $clean
}

tabify() {
	sed -E 's/[ \t]/\t/' < $1 > $1.tab
	mv -f $1.tab $1
}

main() {
	migrate_cache
	set_lock $LOCKFILE_SIGNATURES 60 $FD_SIGNATURES
	if [ "${1:-all}" = 'migrate_only' ]; then
		exit 0
	fi
	for URL in ${autoupdate['url']}; do
		main_loop && break
	done
	release_lock $LOCKFILE_SIGNATURES $FD_SIGNATURES
}

main "$@"
